#!/usr/bin/perl
# ==================================================================
# otpfetch.pl v1.01 - 2004/01/16
#
# PERL program to locally store website images from http://www.thumbshots.org
# and embedding them in a HTML page
# Code is free. No guarantees or warranties.
# See "Metafinder" on http://www.webhits.de for a working implementation
#
# Usage: <img src="/cgi-bin/otpfetch.pl?url=[LINK]">
# ==================================================================

use strict;
use LWP::Simple;
use Digest::MD5;

main: {

# Declare and initialize variables
my $otp = "http://open.thumbshots.org/image.pxf?url=";     # URL prefix for retrieving a thumb
my $default_img = "/path/to/default.gif"; 		   # To use if bad referrer etc. ...
my $local_thumb_dir = "/path/to/local/storage/dir/";	   # Where thumbnail images are stored locally
							   # Web Server needs to be able to write here
my $days_to_keep = 60; 					   # How many days till check if new thumbnail
my @valid_referrer = ('mysite.com','myothersite.com');	   # Your site(s) here

### Nothing to hack for you below ... ;-) ###

# Referrer Checking
my $is_allowed = 0;
my $referrer = "";
if ($ENV{'HTTP_REFERER'}) {
	foreach $referrer (@valid_referrer) {
		if ($ENV{'HTTP_REFERER'} =~ m|https?://([^/]*)$referrer|i) {
			$is_allowed = 1;
			last;
		}
	}
}
else {
	$is_allowed = 1;
}

my @days = ("Sun","Mon","Tue","Wed","Thu","Fri","Sat");
my @months = ("dummy","Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
my $expires = time;
my ($sec,$min,$hour,$mday,$mon,$year,$wday) = gmtime($expires);
$mon++;
$sec = "0" . $sec if $sec < 10;
$min = "0" . $min if $min < 10; 
$hour = "0" . $hour if $hour < 10; 
$mday = "0" . $mday if $mday < 10;
$year += 1900;
my $mod_string = "$days[$wday], $mday $months[$mon] $year $hour:$min:$sec GMT";

$expires += (86400 * $days_to_keep);
($sec,$min,$hour,$mday,$mon,$year,$wday) = gmtime($expires);
$mon++;
$sec = "0" . $sec if $sec < 10;
$min = "0" . $min if $min < 10; 
$hour = "0" . $hour if $hour < 10; 
$mday = "0" . $mday if $mday < 10;
$year += 1900;
my $exp_string = "$days[$wday], $mday $months[$mon] $year $hour:$min:$sec GMT";

my %MYDATA = &get_input($ENV{'QUERY_STRING'});
my $link = $MYDATA{'url'} ;
my $pic_filename = "";

if (($is_allowed == 0) || ($link eq "")) {
	$pic_filename = $default_img;
}
else {
	my $md5 = Digest::MD5->new;
	$md5->add($link, "");
	my $hashed_name = $md5->hexdigest;
	$md5->reset;
	$pic_filename = $local_thumb_dir . $hashed_name . ".jpg";
	if ((! -e $pic_filename) || (-M _ > $days_to_keep)) {
		&fetchit($otp,$link,$pic_filename);
	}
}

&return_image($mod_string, $exp_string, $pic_filename);
exit;

sub return_image {
	my $mod_string = $_[0];
	my $exp_string = $_[1];
	my $pic_filename = $_[2];
	print "Last-Modified: $mod_string\n";
	print "Expires: $exp_string\n";
	print "Content-Type: image/jpeg\n\n";
	open (JPG, "$pic_filename");
	print <JPG>;
	close JPG;
}

sub fetchit {
	my $otp = $_[0];
	my $link = $_[1];
	my $pic_filename = $_[2];
	my $URL = $otp . $link;
	getstore ($URL, $pic_filename);
}

sub get_input {
	my %temp_h;
	my $buffer = $_[0];
	$buffer =~ s/\|/\&/g;
	$buffer =~ s/&amp\;/\&/g;
	my @pairs = split(/&/, $buffer);
	foreach my $pair (@pairs) {
		(my $name, my $value) = split(/=/, $pair);
		$name =~ s/%([0-9|A-F]{2})/pack('C',hex($1))/eg;
		$value =~ s/%([0-9|A-F]{2})/pack('C',hex($1))/eg;
		$temp_h{$name} = $value;
	}
	return %temp_h;
}

} # End main:
